//
// Hough transform -  
// -  
//
//    ,   
//    :)
//
// http://robocraft.ru
//

#include <cv.h>
#include <highgui.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

//
//     
//
//   : http://forum.graphicon.ru/viewtopic.php?p=20222#p20222
//
void houghLine(IplImage* original, float accuracy=0.1)
{
	assert(original!=0);

	IplImage *src=0, *rgb=0;
	IplImage *bin=0;
	IplImage *phase=0;

	src=cvCloneImage(original);

	//   
	rgb = cvCreateImage(cvGetSize(src), 8, 3);
	cvConvertImage(src, rgb, CV_GRAY2BGR);

	//   -  
	bin = cvCreateImage(cvGetSize(src), IPL_DEPTH_8U, 1);
	cvCanny(src, bin, 50, 200);

	cvNamedWindow( "bin", 1 );
	cvShowImage( "bin", bin );

	//      -   
	int RMax = cvRound( sqrt( (double)(src->width*src->width + src->height*src->height) ) );

	//       (r, f)
	// 0 < r < RMax
	// 0 < f < 2*PI
	phase = cvCreateImage(cvSize(RMax, 180), IPL_DEPTH_16U, 1);
	cvZero(phase);

	int x=0, y=0, r=0, f=0;
	float theta=0;

	//     
	for(y=0; y<bin->height; y++){
		uchar* ptr = (uchar*) (bin->imageData + y * bin->widthStep);
		for(x=0; x<bin->width; x++){
			if(ptr[x]>0){ //   ?
				//    ,   
				//    
				for(f=0; f<180; f++){ //    
					short* ptrP = (short*) (phase->imageData + f * phase->widthStep);
					for(r=0; r<RMax; r++){ //       
						theta=f*CV_PI/180.0; //    

						//      (  )
						if ( abs(( (y)*sin(theta) + (x)*cos(theta)) - r) < accuracy ){
							ptrP[r]++; //       .
						}
					}
				}
			}
		}
	}

	cvNamedWindow( "phase", 1 );
	cvShowImage( "phase", phase );

	//   
	IplImage* phaseImage = cvCreateImage( cvSize(phase->width*3, phase->height*3), IPL_DEPTH_16U, 1);
	cvResize(phase, phaseImage);

	cvNamedWindow( "phaseImage", 1 );
	cvShowImage( "phaseImage", phaseImage);

	//         
	unsigned int MaxPhaseValue = 0;
	float Theta=0;
	int R=0;
	for(f=0; f<180; f++){ //    
		short* ptrP = (short*) (phase->imageData + f * phase->widthStep);
		for(r=0; r<RMax; r++){ //       
			if(ptrP[r]>MaxPhaseValue){
				MaxPhaseValue = ptrP[r];
				Theta = f;
				R = r;
			}
		}
	}

#if 1
	printf("[M] %d\n", MaxPhaseValue);

	// 
	float scaler = 0xFFFFFFFF/(float)MaxPhaseValue;
	for(y=0; y<phaseImage->height; y++){
		short* ptr = (short*) (phaseImage->imageData + y * phaseImage->widthStep);
		for(x=0; x<phaseImage->width; x++){
			ptr[x]*=scaler;
		}
	}
	cvNamedWindow( "phaseImage2", 1 );
	cvShowImage( "phaseImage2", phaseImage);
#endif

	//       R, Teta     
	Theta=Theta*CV_PI/180.0;
	for(y=0; y<rgb->height; y++){
		uchar* ptr = (uchar*) (rgb->imageData + y * rgb->widthStep);
		for(x=0; x<rgb->width; x++){
			if ( cvRound(((y) * sin(Theta) + (x) * cos(Theta))) == R){
				ptr[3*x]=0;
				ptr[3*x+1]=0;
				ptr[3*x+2]=255; 
			}
		}
	}

	cvNamedWindow( "line", 1 );
	cvShowImage( "line", rgb );

	//  
	cvReleaseImage(&src);
	cvReleaseImage(&rgb);

	cvReleaseImage(&bin);
	cvReleaseImage(&phase);
	cvReleaseImage(&phaseImage);
}

int main(int argc, char* argv[])
{
	IplImage *original=0;

	//     
	char* filename = argc >= 2 ? argv[1] : "Image0.jpg";
	//  
	original = cvLoadImage(filename, 0);

	printf("[i] image: %s\n", filename);
	assert( original != 0 );

	//  
	cvNamedWindow( "original", 1 );
	cvShowImage( "original", original );

	houghLine(original);

	cvWaitKey(0);

	cvReleaseImage(&original);

	//  
	cvDestroyAllWindows();
	return 0;
}
